


#include "h/p18f4550.h"

#include "h/Affichages.h"
#include "h/I2C.h"
#include "h/Parametres.h"
#include "h/Utilitaires.h"



const rom unsigned char ArrondisValeursPositives[16] = {0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9, 9};
const rom unsigned char ArrondisValeursNegatives[16] = {9, 9, 8, 8, 7, 7, 6, 6, 5, 4, 4, 3, 2, 2, 1, 0};



extern unsigned char g_Affichages[4];

extern unsigned char g_Points_1[4];
extern unsigned char g_Points_2[4];
extern unsigned char g_Points_3[4];

extern unsigned char g_Heures;
extern unsigned char g_Minutes;
extern unsigned char g_Secondes;

extern unsigned char g_Fonction;



void Scrutations(int Transitions1, int Transitions2)
{
	unsigned char k;

	for(k = 0; k < 4; k++)
	{
		PORTA = 0x00;

		PORTC = 0x00;
		PORTD = 0x00;
		PORTE = 0x00;

		PORTB = (1 << (4 + k));

		Delai(Transitions1);

		PORTA = g_Affichages[k];

		PORTC = g_Points_2[k];
		PORTD = g_Points_1[k];
		PORTE = g_Points_3[k];

		Delai(Transitions2);
	}
}



void ScrutationsMultiples(int Phases, int Transitions1, int Transitions2)
{
	int k;

	for(k = 0; k < Phases; k++)
	{
		Scrutations(Transitions1, Transitions2);
	}
}



unsigned char ConvertirPourAffichage(unsigned char Valeur)
{
	switch(Valeur)
	{
		case '0' :
		{
			return 0x77;
			break;
		}
		case '1' :
		{
			return 0x03;
			break;
		}
		case '2' :
		{
			return 0x3D;
			break;
		}
		case '3' :
		{
			return 0x1F;
			break;
		}
		case '4' :
		{
			return 0x4B;
			break;
		}
		case '5' :
		{
			return 0x5E;
			break;
		}
		case '6' :
		{
			return 0x7E;
			break;
		}
		case '7' :
		{
			return 0x07;
			break;
		}
		case '8' :
		{
			return 0x7F;
			break;
		}
		case '9' :
		{
			return 0x5F;
			break;
		}
		case ' ' :
		{
			return 0x00;
			break;
		}
		case '' :
		{
			return 0x4D;
			break;
		}
		case 'A' :
		{
			return 0x6F;
			break;
		}
		case '-' :
		{
			return 0x08;
			break;
		}
		default :
		{
			return 0x00;
			break;
		}
	}
}



void ChangerSecondes(unsigned char Extinction, unsigned char Secondes)
{
	unsigned char k;

	unsigned char Quart;
	unsigned char Ramener;

	if(Extinction == 0)
	{
		if(Secondes == 0)
		{
			g_Points_1[3] = 0x01;
			g_Points_1[2] = 0x00;
			g_Points_1[1] = 0x00;
			g_Points_1[0] = 0x00;
	
			g_Points_2[3] = 0x00;
			g_Points_2[2] = 0x00;
			g_Points_2[1] = 0x00;
			g_Points_2[0] = 0x00;
	
			g_Points_3[3] = 0x00;
			g_Points_3[2] = 0x00;
			g_Points_3[1] = 0x00;
			g_Points_3[0] = 0x00;
		}
		else
		{
			Quart = (Secondes / 15);	/* 0, 1, 2, 3. */
			Ramener = Secondes - (15 * Quart);
		
			if(Ramener > 7)
			{
				g_Points_1[(3 - Quart)] = 0xFF;
				if(Ramener > 10)
				{
					g_Points_2[(3 - Quart)] = 0x03;
					if(Ramener > 12)
					{
						g_Points_2[(3 - Quart)] = 0xC7;
						if(Ramener > 14)
						{
							g_Points_3[(3 - Quart)] = 0x02;
						}
						else
						{
							g_Points_3[(3 - Quart)] = ((1 << (Ramener + 1 - 13)) - 1);	
						}
					}
					else
					{
						g_Points_2[(3 - Quart)] |= ((1 << (Ramener + 1 - 11 + 6)) - 1);	
					}
				}
				else
				{
					g_Points_2[(3 - Quart)] = ((1 << (Ramener + 1 - 8)) - 1);	
				}
			}
			else
			{
				g_Points_1[(3 - Quart)] = ((1 << (Ramener + 1)) - 1);	
			}
		}
	}
	else
	{
		for(k = 0; k < 4; k++)
		{
			g_Points_1[k] = 0x00;
	
			g_Points_2[k] = 0x00;
	
			g_Points_3[k] = 0x00;
		}
	}
}



void ChangerHeuresMinutes(unsigned char Heures, unsigned char Minutes)
{
	if(Heures <= 9)
	{
		g_Affichages[3] = 0x00;
	}
	else
	{
		g_Affichages[3] = ConvertirPourAffichage(0x30 + (Heures / 10));
	}
	g_Affichages[2] = ConvertirPourAffichage(0x30 + (Heures % 10));

	g_Affichages[1] = ConvertirPourAffichage(0x30 + (Minutes / 10));
	g_Affichages[0] = ConvertirPourAffichage(0x30 + (Minutes % 10));
}



void AfficherUneTemperature(unsigned char Endroit, unsigned char L1, unsigned char L2)
{
	unsigned char MasquageIndicationsEndroits = 0x00;

	switch(Endroit)
	{
		case ENDROIT_AFFICHAGE_TEMPERATURE_INTERIEURE :
		{
			MasquageIndicationsEndroits = 0xFF;	/* Si l'affichage de la temprature
												concerne l'intrieur,
												les 12 LED du tour de l'horloge sont teintes. */

			break;
		}

		case ENDROIT_AFFICHAGE_TEMPERATURE_EXTERIEURE :
		{
			MasquageIndicationsEndroits = 0xBF;	/* Si l'affichage de la temprature
												concerne l'intrieur,
												les 12 LED du tour de l'horloge sont allumes. */

			break;
		}

		default :
		{
			MasquageIndicationsEndroits = 0xFF;	/* Si l'affichage de la temprature
												ne concerne ni l'intrieur, ni l'extrieur,
												les 12 LED du tour de l'horloge sont teintes. */

			break;
		}
	}

	if((L1 > 0x79) && (L1 < 0xF6))	/* Si la temprature mesure est infrieure  - 9,9 C */ 
	{
		/* Affichage de l'indication   b    */

		Write(ADRESSE_EXPANSEUR_DE_PORTS, (MasquageIndicationsEndroits & 0xFF));
	
		g_Affichages[3] = 0x00;
		g_Affichages[2] = 0x7A;
		g_Affichages[1] = 0x00;
		g_Affichages[0] = 0x00;
	}
	else
	{
		if((L1 < 0x80) && (L1 > 0x63))	/* Si la temprature mesure est suprieure  99,9 C */
		{
			/* Affichage de l'indication   H    */

			Write(ADRESSE_EXPANSEUR_DE_PORTS, (MasquageIndicationsEndroits & 0xFF));
	
			g_Affichages[3] = 0x00;
			g_Affichages[2] = 0x6B;
			g_Affichages[1] = 0x00;
			g_Affichages[0] = 0x00;
		}
		else
		{
			/* Affichage du symbole virgule -  ,  */
			Write(ADRESSE_EXPANSEUR_DE_PORTS, (MasquageIndicationsEndroits & 0xD7));
			if(L1 < 0x80)
			{
				if(L1 <= 9)
				{
					g_Affichages[3] = 0x00;
				}
				else
				{
					g_Affichages[3] = ConvertirPourAffichage(0x30 + (L1 / 10));
				}
				g_Affichages[2] = ConvertirPourAffichage(0x30 + (L1 % 10));
		
				g_Affichages[1] = ConvertirPourAffichage(0x30 + ((ArrondisValeursPositives[(L2 >> 4)]) % 10));
			}
			else
			{
				L1 = (128 - (0x7F & L1));
		
				/* Affichage du symbole  Moins  -  -  */
				g_Affichages[3] = ConvertirPourAffichage('-');
		
				g_Affichages[2] = ConvertirPourAffichage(0x30 + (L1 - 1));
		
				g_Affichages[1] = ConvertirPourAffichage(0x30 + ((ArrondisValeursNegatives[(L2 >> 4)]) % 10));
			}
			g_Affichages[0] = ConvertirPourAffichage('');
		}	
	}
}



unsigned char IncrementerUneSeconde(unsigned char Heures, unsigned char Minutes, unsigned char Secondes)
{
	unsigned char ChangementsHeuresMinutes = 0;

	if(Secondes < 59)
	{
		g_Secondes++;	
	}
	else
	{
		ChangementsHeuresMinutes = 1;

		g_Secondes = 0;

		if(Minutes < 59)
		{
			g_Minutes++;
		}
		else
		{
			g_Minutes = 0;
			if(Heures < 23)
			{
				g_Heures++;
			}
			else
			{
				g_Heures = 0;
			}
		}
	}

	return ChangementsHeuresMinutes;
}



void RecupererHeure(void)
{
	/* g_Heures = BCDVersOctet(Read16court(0xD0, 0x02, 0xD1));
	g_Minutes = BCDVersOctet(Read16court(0xD0, 0x01, 0xD1));
	g_Minutes = Read16court(0xD0, 0x01, 0xD1);
	g_Secondes = BCDVersOctet(Read16court(0xD0, 0x00, 0xD1));



	Read16nouvelle(0xD0, 0x00, 0xD1, &g_Secondes, &g_Minutes); */



	/* g_Secondes = BCDVersOctet(g_Secondes);
	g_Minutes = BCDVersOctet(g_Minutes); */
	/* g_Secondes = BCDVersOctet(Read16court(0xD0, 0x00, 0xD1)); */



	Read16nouvelle(0xD0, 0x01, 0xD1, &g_Minutes, &g_Heures);



	g_Secondes = Read16court(0xD0, 0x00, 0xD1);

	g_Heures &= 0x3F;

	g_Minutes &= 0x7F;

	g_Secondes &= 0x7F;

	g_Heures = BCDVersOctet(g_Heures);

	g_Minutes = BCDVersOctet(g_Minutes);

	g_Secondes = BCDVersOctet(g_Secondes);


}


